"use strict";

{
	C3.Plugins.aaXe_RotJs.Exps =
	{
		Seed()
		{
			return this._seed;
		},

		GetUniform()
		{
			return ROT.RNG.getUniform();
		},

		GetNormal(mean, stddev)
		{
			return ROT.RNG.getNormal(mean, stddev);
		},

		GetPercent()
		{
			return ROT.RNG.getPercentage();
		},

		GetWeighted(list)
		{
			var items = list.split(",");
			var obj = {};

			items.forEach((ele) => {
				var item = ele.split(":");
				obj[item[0].trim()] = Number(item[1].trim()); 
			});

			return ROT.RNG.getWeightedValue(obj);
		},

		RoomCount()
		{
			return this.rooms.length;
		},

		IslandCount()
		{
			return this.blobs.length;
		},

		CurIslandIndex()
		{
			return this.curBlobIndex;
		},

		//Room iteration Expressions
		CurRoomIndex()
		{
			return this.curRoomIndex;
		},

		CurRoomHeight()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.height;
		},

		CurRoomWidth()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.width;
		},

		CurRoomTop()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.top;
		},

		CurRoomBottom()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.bottom;
		},

		CurRoomRight()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.right;
		},

		CurRoomLeft()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.left;
		},

		CurRoomCenterX()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.centerX;
		},

		CurRoomCenterY()
		{
			if(!this.curRoom) return -1;
			return this.curRoom.centerY;
		},

		CurRoomDoorCount()
		{
			if(!this.curRoom || !this.curRoom.doors) return -1;
			return this.curRoom.doors.length;
		},

		CurRoomFreeCellCount()
		{
			if(!this.curRoom || !this.curRoom.cells) return -1;
			return this.curRoom.cells.length;
		},

		//general room access expressions
		RoomHeight(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].height;
		},

		RoomWidth(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].width;
		},

		RoomTop(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].top;
		},

		RoomBottom(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].bottom;
		},

		RoomRight(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].right;
		},

		RoomLeft(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].left;
		},

		RoomCenterX(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].centerX;
		},

		RoomCenterY(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			return this.rooms[roomIndex].centerY;
		},

		RoomDoorCount(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			if(!this.rooms.doors) return -1;
			return this.rooms[roomIndex].doors.length;
		},

		RoomFreeCellCount(roomIndex)
		{
			if(!this.rooms) return -1;
			if(this.rooms.length <= roomIndex) return -1;
			if(!this.rooms.cells) return -1;
			return this.rooms[roomIndex].cells.length;
		},

		//iterrate doors expressions
		CurDoorX()
		{
			if(!this.curDoor) return -1;
			return this.curDoor.x;
		},

		CurDoorY()
		{
			if(!this.curDoor) return -1;
			return this.curDoor.y;
		},

		CurDoorRoomIndex()
		{
			if(!this.curDoor) return -1;
			return this.curDoor.room;
		},

		//general door access expressions
		DoorX(doorIndex)
		{
			if(!this.doors) return -1;
			if(this.doors.length <= doorIndex) return -1;
			return this.doors[doorIndex].x;
		},

		DoorY(doorIndex)
		{
			if(!this.doors) return -1;
			if(this.doors.length <= doorIndex) return -1;
			return this.doors[doorIndex].y;
		},

		DoorRoomIndex(doorIndex)
		{
			if(!this.doors) return -1;
			if(this.doors.length <= doorIndex) return -1;
			return this.doors[doorIndex].room;
		},

		//iterate cell expressions
		CurCellX()
		{
			if(!this.curCell) return -1;
			return this.curCell.x;
		},

		CurCellY()
		{
			if(!this.curCell) return -1;
			return this.curCell.y;
		},

		CurCellValue()
		{
			if(!this.curCell) return -1;
			return this.curCell.value;
		},

		CellValue(x, y)
		{
			if(!this.map) return -1;
			if(!this.map.hasOwnProperty(x+','+y)) return -1;
			return this.map[x+','+y].value;
		},

		CurCellType()
		{
			if(!this.curCell) return -1;
			return this.curCell.type;
		},
		CellType(x,y)
		{
			if(!this.map) return -1;
			if(!this.map.hasOwnProperty(x+','+y)) return -1;
			return this.map[x+','+y].type;
		},

		//fov expressions
		FovCellIndex()
		{
			return this.curFovIndex;
		},
		FovCellX()
		{
			if(!this.curFovCell) return -1;
			return this.curFovCell.x;
		},
		FovCellY()
		{
			if(!this.curFovCell) return -1;
			return this.curFovCell.y;
		},
		FovCellRadius()
		{
			if(!this.curFovCell) return -1;
			return this.curFovCell.radius;
		},
		FovCellVisibility(){
			if(!this.curFovCell) return -1;
			return this.curFovCell.visibility;
		},

		//import /export expression
		AsJson()
		{
			var data = this.SaveToJson();
			return JSON.stringify(data);

		}
	};
}